from unittest import TestCase


from pyutls import euclidean_division


class TestNumber(TestCase):

    def test_euclidean_division(self):
        numerator = 15
        denominator = 2
        quotient, remainder = euclidean_division(numerator, denominator)
        self.assertEqual(quotient, 7)
        self.assertEqual(remainder, 1)
        self.assertEqual(quotient * denominator + remainder, numerator)
        with self.assertRaises(Exception):
            euclidean_division(numerator, denominator=0)



import math


def euclidean_division(numerator: int, denominator: int) -> tuple[int, int]:
    """Compute the euclidean division of the passed integers"""
    if denominator == 0:
        raise Exception("division by zero")
    quotient = math.floor(numerator / denominator)
    remainder = numerator % denominator
    return quotient, remainder



import logging
import os
import sys
from datetime import datetime, timezone


DEFAULT_NOISY_LOGGERS = [
    "asyncio",
    "boto3",
    "botocore",
    "httpcore",
    "httpx",
    "requests",
    "s3transfer",
]


class CustomLogger:
    """
    CustomLogger provides centralized logging setup with optional suppression
    of noisy third-party loggers and a convenience method for timestamped logging.

    Args:
        noisy_loggers (list[str], optional): List of logger names to suppress (set to ERROR level).
        level (int, optional): The global logging level. Defaults to logging.INFO.
        logger_file (str, optional): File name for saving logs. Defaults to 'logger.log'.
                                     In AWS Lambda environments, it is automatically redirected to '/tmp/'.

    Example:
        >>> logger = CustomLogger(
        ...     noisy_loggers=["boto3", "httpx", "requests"],
        ...     level=logging.INFO
        ... )
        >>> logger.with_time("Application started")
        >>> logger.with_time("An error occurred", lvl=logging.ERROR)

    Notes:
        - Only use `logging.ERROR` level in `except:` blocks.
        - Use `logging.CRITICAL` for custom fatal error conditions.
        - The `flush=True` flag can be used to add spacing between log entries in terminal output.
    """

    def __init__(
        self, noisy_loggers=None, level=logging.INFO, logger_file="logger.log"
    ):
        if os.getenv("AWS_EXECUTION_ENV"):
            logger_file = f"/tmp/{logger_file}"

        logging.basicConfig(
            format="%(asctime)s - %(levelname)s - %(message)s",
            level=level,
            handlers=[
                logging.FileHandler(logger_file),
                logging.StreamHandler(sys.stdout),
            ],
        )

        # Use ISO-formatted local time
        logging.Formatter.formatTime = (
            lambda self, record, datefmt=None: datetime.fromtimestamp(
                record.created, timezone.utc
            )
            .astimezone()
            .isoformat(" ")
        )

        # Suppress noisy loggers
        if noisy_loggers is None:
            noisy_loggers = DEFAULT_NOISY_LOGGERS
        self.noisy_loggers = noisy_loggers or []
        for logger_name in self.noisy_loggers:
            logging.getLogger(logger_name).setLevel(logging.ERROR)

    def with_time(self, msg, lvl=logging.INFO, flush=False):
        """
        Log message to both stdout and file.

        Args:
            msg (str): Message to log.
            lvl (int): Logging level (e.g., logging.INFO, logging.ERROR). Defaults to logging.INFO.
            flush (bool): If True, adds a newline before the message for better spacing. Defaults to False.

        Usage:
            >>> logger.with_time("Something happened", lvl=logging.WARNING)
            >>> logger.with_time("New section", flush=True)
        """
        if flush:
            print("", end="\n", flush=True)

        if lvl == logging.ERROR:
            logging.exception(msg)
        elif lvl == logging.CRITICAL:
            logging.critical(msg)
        elif lvl == logging.WARNING:
            logging.warning(msg)
        elif lvl == logging.DEBUG:
            logging.debug(msg)
        else:
            logging.info(msg)



def chunk(items, size):
    """Split a list into chunks of a maximum size"""
    return [items[i: i + size] for i in range(0, len(items), size)]


def flatten(t):
    """Flatten a list of list"""
    return [item for sublist in t for item in sublist]



from unittest import TestCase


from pyutls import CustomLogger


class TestLogger(TestCase):

    def test_logger(self):
        logger = CustomLogger()
        logger.with_time("This test line should appear in test logs!")



from unittest import TestCase


from pyutls import from_hex, to_hex


class TestHex(TestCase):

    def test_hex(self):
        ref = "1234abcd"
        barray = from_hex(ref)
        self.assertIsNotNone(barray)
        hex = to_hex(barray)
        self.assertEqual(hex, ref)

        wrong = from_hex("not-hex-string")
        self.assertIsNone(wrong)



from unittest import TestCase


from pyutls import flatten, chunk


class TestChunk(TestCase):

    def test_chunk(self):
        items = [1, 2, 3, 4, 5]
        chunks = chunk(items, 2)
        self.assertEqual(3, len(chunks))
        self.assertListEqual(chunks[0], [1, 2])
        self.assertListEqual(chunks[1], [3, 4])
        self.assertListEqual(chunks[2], [5])


class TestFlatten(TestCase):

    def test_flatten(self):
        one = ["one"]
        two = ["two"]
        both = [one, two]
        flat = flatten(both)
        self.assertEqual(flat, ["one", "two"])

        some_list = [1, 2, 3, 4, 5]
        chunks = chunk(some_list, 2)
        flat_list = flatten(chunks)
        self.assertEqual(some_list, flat_list)



from typing import Optional


def from_hex(string: str) -> Optional[bytes]:
    """Builds some byte array from the passed hexadecimal string or None if not appropriate"""
    try:
        return bytes.fromhex(string)
    except:
        return None


def to_hex(barray: bytes) -> str:
    """Returns the hexadecimal representation of the passed byte array"""
    return "".join("{:02x}".format(x) for x in barray)
